<?php
/**
 * @package     BreezingCommerce
 * @author      Markus Bopp
 * @link        http://www.crosstec.de
 * @license     GNU/GPL
*/
defined( '_JEXEC' ) or die( 'Restricted access' );

$libpath = JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce' . DS . 'classes' . DS . 'plugin' . DS;
require_once($libpath . 'CrBcAShippingSitePlugin.php');
require_once($libpath . 'CrBcShippingSitePlugin.php');

class CrBc_Plugins_Shipping_Distance_Site extends CrBcAShippingSitePlugin implements CrBcShippingSitePlugin
{
    private $cart = null;
    
    public function  __construct() {
        
        require_once(JPATH_COMPONENT_ADMINISTRATOR.DS.'classes'.DS.'CrBcCart.php');
        
        // always call the parent constructor and always call it _first_
        parent::__construct();
        // define the default table for built-in list/details view
        $this->table = '#__breezingcommerce_plugin_distance';
    }
    
    function init($subject = NULL){
        if( !($subject instanceof CrBcCart) ){ throw new Exception('init subject must be of type CrBcCart'); }
        $this->cart = $subject;
    }
    
    function getOptionElementName(){
        return 'crbc_shipping_distance';
    }
    
    public function getPluginDisplayName(){
        return JText::_('COM_BREEZINGCOMMERCE_PLUGIN_DISTANCE');
    }
    
    function validateSelectedOption($option, $order_id){
        if(intval($option) <= 0){
            return JText::_('COM_BREEZINGCOMMERCE_PLUGIN_DISTANCE_SHIPPING_PLEASE_SELECT_AN_OPTION');
        }
        return '';
    }
    
    function onOptionUpdate($option, $order_id){
        // nothing required here
    }
    
    function determineBestShippingOptions(){
        // nothing, distance doesn't care
    }
    
    function onDisplayOptions($selected_key){

        $db = JFactory::getDBO();
        $db->setQuery("Select * From #__breezingcommerce_plugin_distance Where published = 1 Order By `rate`");
        $options = $db->loadAssocList();
        $the_option = null;
        $remote_country = '';
        $remote_city = '';
        $business_country = '';
        $business_city = '';

        // we are in cart, so we try to estimate
        if(isset($_REQUEST['controller']) && isset($_REQUEST['view']) && $_REQUEST['controller'] == 'cart' && $_REQUEST['view'] == 'cart'){

            $ch = @curl_init();
            @curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            @curl_setopt($ch, CURLOPT_URL,'http://ip-api.com/json/' . $this->get_client_ip());
            // @curl_setopt($ch, CURLOPT_URL,'http://ip-api.com/json/2A02:908:392:2100:14C2:D692:35C5:7678');
            $result = @curl_exec($ch);
            @curl_close($ch);
            $result = @json_decode($result, true);

            if(is_array($result)) {

                $remote_country = isset($result['country']) ? $result['country'] : '';
                $remote_city = isset($result['city']) ? $result['city'] : '';
            }

        }else{

            // try to pull country and city from customer, assuming we are on checkout
            $db->setQuery("Select * From #__breezingcommerce_customers Where id = " . $db->quote( $this->cart->cart['customer_id'] ));
            $customer = $db->loadObject();

            if($customer){

                if($customer->use_shipping_address == 1){

                    $remote_country = $customer->shipping_country_id;
                    $remote_city = $customer->shipping_city;
                }
                else{
                    $remote_country = $customer->country_id;
                    $remote_city = $customer->city;
                }
            }

            if($remote_country){

                $db->setQuery("Select `name` From #__breezingcommerce_countries Where id = " . $db->quote( $remote_country) );
                $remote_country = $db->loadResult();
            }
        }

        $db->setQuery("Select attribs From #__breezingcommerce_configuration");
        $config = $db->loadResult();

        if($config){

            $config = json_decode($config);
            $business_country = $config->business_country;
            $business_city = $config->business_city;
        }

        if($remote_city == '' || $remote_city == '' || $business_city == '' || $business_country == ''){

            // error

        }else{

            $ch = @curl_init();
            @curl_setopt($ch,CURLOPT_RETURNTRANSFER, true);
            @curl_setopt($ch,CURLOPT_PROXY_SSL_VERIFYPEER, 0);
            @curl_setopt($ch,CURLOPT_PROXY_SSL_VERIFYHOST, 0);
            @curl_setopt($ch,CURLOPT_CAINFO,__DIR__ . '/cacert.pem');
            @curl_setopt($ch,CURLOPT_CAPATH,__DIR__ . '/cacert.pem');

            if($remote_country == 'United States' || $remote_country == 'United States of America'){

                $remote_country = 'USA';
            }

            //@curl_setopt($ch,CURLOPT_URL,'https://www.entfernung.org/route.json?stops=' . urlencode($business_city . ', ' . $business_country . '|' . $remote_city . ', ' . $remote_country));
            @curl_setopt($ch,CURLOPT_URL,'https://www.entfernung.org/route.json?stops=' . urlencode($business_city . '|' . $remote_city));

            $result = @curl_exec($ch);
            @curl_close($ch);
            $result = @json_decode($result, true);

            if($result && isset($result['distance']) && isset($result['stops']) && $result['stops'][0]['type'] != 'Invalid' && $result['stops'][1]['type'] != 'Invalid'){

                if(count($options)){

                    foreach($options As $option){

                        $from_distance = $option['from_distance'];
                        $to_distance = $option['to_distance'];

                        if($option['km_miles'] == 'miles'){

                            $from_distance = $from_distance * 1.60934;
                            $to_distance = $to_distance * 1.60934;

                        }

                        if(  $result['distance'] >= $from_distance && $result['distance'] < $to_distance || (  $result['distance'] >= $from_distance && $to_distance == 0 ) ){

                            $the_option = $option;
                            break;
                        }
                    }
                }
            }
            else{

                foreach($options As $option){

                    $to_distance = $option['to_distance'];

                    if(  $to_distance == 0 ){

                        $the_option = $option;
                        break;
                    }
                }
            }
        }

        if(!$the_option){

            // error
        }
        else{

            CrBcCart::$exclude_from_no_tax_list[] = $the_option['producttaxclass_id'];
            $result = CrBcCart::getPrice($the_option['rate'], $this->cart->currency_conversion_rate, $the_option['producttaxclass_id'], 1, $this->cart->cart['customer_id']);

            $translated_object = CrBcHelpers::loadTranslation($the_option['identity'], 'plugin_shipping_distance');
            if($translated_object !== null){
                $option['key'] = $translated_object->title;
            }

            $text = JText::_('COM_BREEZINGCOMMERCE_PLUGIN_DISTANCE_BASED_ON');
            if(isset($_REQUEST['controller']) && isset($_REQUEST['view']) && $_REQUEST['controller'] == 'cart' && $_REQUEST['view'] == 'cart'){

                $text = JText::_('COM_BREEZINGCOMMERCE_PLUGIN_DISTANCE_BASED_ON_CART');
            }

            $out = $the_option['key'].' ('.$this->cart->formatPrice($result['gross']).')';
            $out .= '<div><i class="crbc-fa crbc-fa-truck"></i> '.$text.'</div>';
            return $out . '<input type="hidden" value="'.$the_option['identity'].'" name="'.$this->getOptionElementName().'" />';
        }

        return '';
    }
    
    function getProductTaxclassId($option){
       $db = JFactory::getDBO();
       $db->setQuery("Select producttaxclass_id From #__breezingcommerce_plugin_distance Where published = 1 And identity = ".intval($option));
       $result = $db->loadResult();
       if($result){
           return $result;
       }
       return 0;
    }
    
    function getPrice($option){
       $db = JFactory::getDBO();
       $db->setQuery("Select rate From #__breezingcommerce_plugin_distance Where published = 1 And identity = ".intval($option));
       $result = $db->loadResult();
       if($result){
           return $result * $this->cart->currency_conversion_rate;
       }
       return 0;
    }
    
    function getName($option){
       $db = JFactory::getDBO();
       $db->setQuery("Select `key` From #__breezingcommerce_plugin_distance Where published = 1 And identity = ".intval($option));
       $result = $db->loadResult();
       if($result){
           return $result;
       }
       return JText::_('COM_BREEZINGCOMMERCE_PLUGIN_DISTANCE_UNKNOWN');
    }

    function get_client_ip() {
        $ipaddress = '';
        if (getenv('HTTP_CLIENT_IP'))
            $ipaddress = getenv('HTTP_CLIENT_IP');
        else if(getenv('HTTP_X_FORWARDED_FOR'))
            $ipaddress = getenv('HTTP_X_FORWARDED_FOR');
        else if(getenv('HTTP_X_FORWARDED'))
            $ipaddress = getenv('HTTP_X_FORWARDED');
        else if(getenv('HTTP_FORWARDED_FOR'))
            $ipaddress = getenv('HTTP_FORWARDED_FOR');
        else if(getenv('HTTP_FORWARDED'))
            $ipaddress = getenv('HTTP_FORWARDED');
        else if(getenv('REMOTE_ADDR'))
            $ipaddress = getenv('REMOTE_ADDR');
        else
            $ipaddress = 'UNKNOWN';
        return $ipaddress;
    }
}